#!/bin/bash

##################################################################
#
# Migrate SLES-12 to SLES_SAP-12 
#
# Copyright (C) 2018 SUSE Linux GmbH
# Author: Stefan Weiberg <sweiberg@suse.com>
#
##################################################################

ARCH=$( uname -m )

# Check if root runs this script
if [[ $EUID -ne 0 ]]; then
   echo -e "\nThis script must be run as root\n" 1>&2
   exit 1
fi
echo -e "\nThis script will migrate your installed\n
\tSUSE Linux Enterprise Server 12 SP4\n
to a\n
\tSUSE Linux Enterprise Server for SAP Applications 12 SP4.\n
Do you want to continue? [y/n]\n"

. /etc/os-release

if [ "${CPE_NAME}" != "cpe:/o:suse:sles:12:sp4" ]; then
    echo -e "\nThis is an installation of ${PRETTY_NAME}"
    echo -e "\nMigration is only supported for plain SUSE Linux Enterprise Server 12 SP4\n"
    exit 1
fi

start_migration() {

    read ANSWER

    case $ANSWER in
        y)
            echo -e "\nNow starting the migration...\n"
        ;;  
        n)
            echo -e "\nExiting.\n"
            exit 0
        ;;  
        *)
            echo -e "\nInvalid answer. Please enter either 'y' or 'n'.\n"
            start_migration
        ;;  
    esac
}

start_migration

echo -e "\nThis script can use a local SMT server\n
instead of using SCC for the migration.\n
Do you want to use a local SMT server? [y/n]\n"

use_smt() {

    read ANSWER

    case $ANSWER in
        y)
            USE_SMT=y
            echo -e "\nPlease enter the https URL to the SMT server
            Example: https://mysmt.corp.com:\n"
            read SMT_URL

            #Get SMT certificate and clientSetup4SMT.sh
            wget --no-verbose -q --no-check-certificate --dns-timeout 10 --connect-timeout 10 --output-document /tmp/smt.crt "$SMT_URL"/smt.crt
            if [ $? -ne 0 ]; then
               echo "\nUnable to contact SMT server. Exiting.\n"
               exit 1
            fi
            wget --no-verbose -q --no-check-certificate --dns-timeout 10 --connect-timeout 10 --output-document /tmp/clientSetup4SMT.sh "$SMT_URL"/repo/tools/clientSetup4SMT.sh
            chmod +x /tmp/clientSetup4SMT.sh

            #Gather fingerprint to automatically register to SMT
            FINGERPRINT=$(openssl x509 -in /tmp/smt.crt -fingerprint | grep "^SHA1 Fingerprint=" | cut -d'=' -f2)
        ;;
        n)
            USE_SMT=n
            echo -e "\nPlease enter the email address to be used to register
            SUSE Linux Enterprise Server for SAP Applications 12 SP4:\n"
            read EMAIL_ADDRESS

            echo -e "\nPlease enter your activation code for
            SUSE Linux Enterprise Server for SAP Applications 12 SP4:\n"
            read ACTIVATION_CODE
        ;;
        *)
            echo -e "\nInvalid answer. Please enter either 'y' or 'n'.\n"
            use_smt
        ;;
    esac
}

use_smt

# Call 'SUSEConnect' to subscribe to SLES-12 chanels
if [ -x /usr/sbin/SUSEConnect ] ; then
   if [ "$USE_SMT" = "y" ] ; then
       /tmp/clientSetup4SMT.sh "$SMT_URL"/center/regsvc --fingerprint "$FINGERPRINT" --yes > /dev/null 2>&1
       # This command works but generates an error so no error checking
       /usr/sbin/SUSEConnect -p SLES_SAP/12.4-0/${ARCH} --url "$SMT_URL" > /dev/null 2>&1
       rpm -e --nodeps sles-release > /dev/null 2>&1
   else
       rpm -q sles-release && rpm -e --nodeps sles-release
       /usr/sbin/SUSEConnect -p SLES_SAP/12.4-0/${ARCH} -e "$EMAIL_ADDRESS" -r "$ACTIVATION_CODE" 
       if [ $? -ne 0 ] ; then
           echo -e "\nThe registration was NOT successful."
           echo -e "\nPlease check your credentials or contact support.\n"

           echo -e "\nExiting.\n"
           exit 1
       fi
   fi
else
    echo -e "\n/usr/sbin/SUSEConnect is either not available or not executable.\n"
    echo -e "\nPlease install the package 'SUSEConnect' or ensure /usr/sbin/SUSEConnect is executable.\n"

    echo -e "\nExiting.\n"
    exit 1
fi

# The release files need to be exchanged
echo -e "\nExchanging release packages...\n"
zypper -n install -f SLES_SAP-release sles-release > /dev/null 2>&1
rpm -e sles-release-DVD > /dev/null 2>&1

# Adjust /boot/grub2/grub.cfg:
# --> Alter the titles of the standard and failsafe sections to display 
cp /etc/default/grub /etc/default/grub-SLES12
sed "s/SLES12/SLES_SAP-12/" -i /etc/default/grub
/sbin/update-bootloader --reinit

#Clean up SLES Repos:
if [ -e /etc/zypp/repos.d/SLES12-12-4.repo ] ; then
   rm /etc/zypp/repos.d/SLES12-12-4.repo
fi
if [ -e /etc/zypp/services.d/SUSE_Linux_Enterprise_Server_12_SP4_${ARCH}.service ] ; then
   rm /etc/zypp/services.d/SUSE_Linux_Enterprise_Server_12_SP4_${ARCH}.service
fi
for i in /etc/zypp/repos.d/SUSE_Linux_Enterprise_Server_12_SP4_${ARCH}*
do
   if [ -e "$i" ] ; then
      rm $i
   fi
done

#Cleanup files
rm -f /tmp/smt.crt* /tmp/clientSetup4SMT.sh

echo -e "\nYou're now subscribed to following channels:\n"
zypper lr

# EOF